from __future__ import unicode_literals

import csv
import random
from os import rename
import importlib



from django.core.management.base import BaseCommand
from django.contrib.auth.models import User, Group
from django.utils.text import slugify
from django.contrib.auth import authenticate

from sports.models import Sport, Season, Team, Player, Game, GamePeriod, League, Venue, Division, Broadcast

class Command(BaseCommand):
    help = """Loads Participants from a csv file and creates Club based on parsed file name with Participants"""

    def handle(self, *args, **options):
       Game.objects.all().delete()
       GamePeriod.objects.all().delete()
       # Sport.objects.all().delete()
       # Player.objects.all().delete()
       # Season.objects.all().delete()
       # League.objects.all().delete()
       # Team.objects.all().delete()
       # Venue.objects.all().delete()
       # Division.objects.all().delete()
       # Broadcast.objects.all().delete()


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import json
from argparse import Namespace
from dateutil.parser import parse

from django.core.management.base import BaseCommand
from django.utils.lru_cache import lru_cache

from ...models import Broadcast, Game, GamePeriod, Season, Team, Venue


class Command(BaseCommand):
    help = """Loads NFL Game Period data from JSON."""

    def add_arguments(self, parser):
        parser.add_argument('file_name', nargs='+', type=str)

    def handle(self, *args, **options):
        file_name = options['file_name'][0]
        file_json = open(file_name).read()

        season = json.loads(file_json, object_hook=lambda d: Namespace(**d))
        season_obj, _ = Season.objects.get_or_create(
            season_name=season.year, season_type=season.type, label=self.derive_label_from_file_name(file_name))

        for game_period in season.weeks:
            game_period_obj, created = GamePeriod.objects.update_or_create(
                sport_radar_period_id=game_period.id, defaults={'number': game_period.sequence, 'season': season_obj})
            self.process_games(game_period.games, game_period_obj)

    @classmethod
    def derive_label_from_file_name(cls, file_name):
        try:
            file_name = (((file_name.split('/'))[-1]).split('.')[0]).replace('-', ' ')
            file_name = file_name.replace(' games', '')
            return file_name
        except:
            return file_name

    @classmethod
    def get_venue_from_parent(cls, parent):
        """
        Attempts to extract a Venue object from the given «obj». If one is
        found, update or create the venue and return the resulting object.
        Otherwise, return None.

        :return: Venue or None
        :rtype: Venue or None
        """
        venue = getattr(parent, 'venue', None)
        if not venue:
            return None

        venue_obj, created = Venue.objects.update_or_create(
            venue_id=venue.id, defaults={
                'name': venue.name,
                'country': getattr(venue, 'country', ''),
                'city': getattr(venue, 'city', ''),
                'state': getattr(venue, 'state', ''),
                'zip_code': getattr(venue, 'zip', ''),
            }
        )
        print('{0} venue: {1}'.format('Created' if created else 'Updated', venue_obj))
        return venue_obj

    @classmethod
    @lru_cache()
    def team_from_id(cls, team_id):
        team_obj, created = Team.objects.get_or_create(team_id=team_id)
        if created:
            team_obj.name = team_id
            team_obj.save()
            print('{0} team: {1}'.format('Created' if created else 'Updated', team_obj))
        return team_obj

    @classmethod
    def process_games(cls, game_list, game_period_obj):
        deleteable_broadcast_ids = []
        for game in game_list:
            game_obj, created = Game.objects.update_or_create(
                game_id=game.id, defaults={
                    'game_period': game_period_obj,
                    'scheduled': parse(game.scheduled),
                    'home': cls.team_from_id(game.home.id),
                    'away': cls.team_from_id(game.away.id),
                    'status': game.status,
                    'venue': cls.get_venue_from_parent(game),
                }
            )

            if getattr(game, 'broadcast', False):
                # Creates new Broadcast objects as required
                broadcast_ids = []
                for broadcast_type, broadcast_address in game.broadcast.__dict__.items():
                    broadcast_obj, created = Broadcast.objects.get_or_create(
                        game_id=game_obj.pk, broadcast_type=broadcast_type,
                        broadcast_address=broadcast_address)
                    broadcast_ids.append(broadcast_obj.pk)
            else:
                broadcast_ids = []

            # Which broadcast objects for this game, if any, can be deleted?
            deleteable_broadcast_ids.extend(Broadcast.objects.exclude(id__in=broadcast_ids)
                                                             .filter(game_id=game_obj.pk)
                                                             .values_list('pk', flat=True))
        # Now, delete all the unused broadcast objects in one go
        Broadcast.objects.filter(id__in=deleteable_broadcast_ids).delete()



# Generated by Django 5.0 on 2024-01-13 19:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0015_season_hash'),
    ]

    operations = [
        migrations.AlterField(
            model_name='venue',
            name='address',
            field=models.CharField(blank=True, default=None, max_length=128, null=True, verbose_name='address'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='city',
            field=models.CharField(blank=True, default=None, max_length=32, null=True, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='country',
            field=models.CharField(blank=True, default=None, max_length=32, null=True, verbose_name='country'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='name',
            field=models.CharField(max_length=128, verbose_name='name'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='state',
            field=models.CharField(blank=True, default=None, max_length=32, null=True, verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='venue_id',
            field=models.CharField(max_length=40, verbose_name='venue ID'),
        ),
        migrations.AlterField(
            model_name='venue',
            name='zip_code',
            field=models.CharField(blank=True, default=None, max_length=10, null=True, verbose_name='ZIP Code'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime
import os
import requests

from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils.encoding import smart_str


class Command(BaseCommand):
    help = """Pulls data from sportsradar API."""
    season_config = {}

    global_values = {
        'ncaa-divisions': ['FBS', 'FCS', 'D2', 'D3', 'NAIA', 'USCAA'],
        'types': ['teams', 'games']
    }
    season_config['Global'] = global_values

    # NCAA Config - move this to a settings file eventually
    ncaa_config = {
        'api_key': 'g56be5vwvehput2zr3fjmejg',
        'format': 'json',
        'access_level': 't',
        'division': ['FBS', 'FCS', 'D2', 'D3',  'NAIA', 'USCAA'],
        'version': '1',
        'https': True,
    }
    action_urls = {}

    # specify the exact API url as listed in the SportRadar docs, along with
    # guide in comments below.
    # http://developer.sportradar.us/docs/NCAA_Football_API

    # Grab Teams
    action_urls.update({
        'teams':
        'http(s)://api.sportradar.us/ncaafb-[access_level][version]/teams/[division]/hierarchy.[format]'
        '?api_key=[your_api_key]'
    })

    # [access_level] = Real - Time(rt), Premium(p), Standard(s), Basic(b), Trial(t)
    # [version] = whole number(sequential, starting with the number 1)
    # [division] = FBS, FCS, D2, D3, NAIA or USCAA
    # [format] = xml, json

    # Grab Games
    action_urls.update({
        'games':
        'http(s)://api.sportradar.us/ncaafb-[access_level][version]/[year]/[ncaafb_season]/schedule.[format]'
        '?api_key=[your_api_key]'
    })

    # [access_level] = Real - Time(rt), Premium(p), Standard(s), Basic(b), Trial(t)
    # [version] = whole number(sequential, starting with the number 1)
    # [year] = yyyy
    # [ncaafb_season] = Regular Season(REG), Postseason(PST) - Note: PST used
    #                   for seasons 2012 and prior
    # [format] = xml, json

    ncaa_config.update({'action_urls': action_urls})
    season_config['NCAA-Reg'] = ncaa_config

    # NFL Config - move this to a settings file eventually
    nfl_config = {
        'api_key': 'dj2c5dpkcq6c9z6k38btaarr',
        'format': 'json',
        'access_level': 't',
        'division': ['FBS', 'FCS', 'D2', 'D3',  'NAIA', 'USCAA'],
        'version': '1',
        'https': True,
    }
    action_urls = {}

    # specify the exact API url as listed in the SportRadar docs, along with
    # guide in comments below.
    # http://developer.sportradar.us/docs/NCAA_Football_API

    # Grab Teams
    action_urls.update({
        'teams':
        'http(s)://api.sportradar.us/ncaafb-[access_level][version]/teams/[division]/hierarchy.[format]'
        '?api_key=[your_api_key]'
    })

    # [access_level] = Real - Time(rt), Premium(p), Standard(s), Basic(b), Trial(t)
    # [version] = whole number(sequential, starting with the number 1)
    # [division] = FBS, FCS, D2, D3, NAIA or USCAA
    # [format] = xml, json

    # Grab Games
    action_urls.update({
        'games':
        'http(s)://api.sportradar.us/ncaafb-[access_level][version]/[year]/[ncaafb_season]/schedule.[format]'
        '?api_key=[your_api_key]'
    })

    # [access_level] = Real - Time(rt), Premium(p), Standard(s), Basic(b), Trial(t)
    # [version] = whole number(sequential, starting with the number 1)
    # [year] = yyyy
    # [ncaafb_season] = Regular Season(REG), Postseason(PST) - Note: PST used
    #                   for seasons 2012 and prior
    # [format] = xml, json

    ncaa_config.update({'action_urls': action_urls})
    season_config['NCAA-Reg'] = ncaa_config

    def add_arguments(self, parser):
        # allow pulling of previous years for testing
        years = range(2012, datetime.datetime.now().year + 1)
        division = ['FBS', 'FCS', 'D2', 'D3',  'NAIA', 'USCAA']
        choices = self.season_config.keys()
        parser.add_argument('season', nargs='+', type=str, help='The Season, eg NFL or NCAA', choices=choices)
        parser.add_argument('year', nargs='+', type=int, help='The Year, eg 2016', choices=years)
        parser.add_argument('division', nargs='+', type=str, help='The Division, eg. FSB', choices=division)
        parser.add_argument('type', nargs='+', type=str, help='The Year, eg 2016', choices=('teams','games'))

    def handle(self, *args, **options):
        season = str(options['season'][0])
        year = str(options['year'][0])
        division = str(options['division'][0])
        season_type = str(options['type'][0])
        data_format = str(self.season_config[season]['format'])

        # Generate the filename from the season, year, division and type, in
        # preferred format
        file_name = "{season}-{year}-{division}-{season_type}.{data_format}".format(
            season=season,
            year=year,
            division=division,
            season_type=season_type,
            data_format=data_format,
        )
        url = self.get_request_url(
            config=self.season_config,
            season=season,
            year=year,
            division=division,
            season_type=season_type,
        )

        # send the request and receive the response in text format
        response = self.send_request(url)

        # Make sure the data dir, as specified in the default django settings
        # file exists.
        target_dir = "{dir}{year}/{season_type}/".format(
            dir=getattr(settings, 'SPORTSDATADIR', 'data/'),
            year=year,
            season_type=season_type,
        )
        self.ensure_dir(target_dir)
        self.write_response_to_disk(target_dir + file_name, response)
        return url

    @classmethod
    def ensure_dir(cls, f):
        d = os.path.dirname(f)
        if not os.path.exists(d):
            os.makedirs(d)

    @classmethod
    def write_response_to_disk(cls, filename, text):
        with open(filename, mode='w') as out_file:
            out_file.write(text)

    @classmethod
    def get_request_url(cls, config, season, year, division, season_type):
        """
        Returns the appropriate URL for the requested data.
        """
        is_https = config[season]['https']
        url = str((config[season]['action_urls'])[season_type])

        placeholders = {
            'http(s)': 'https' if is_https else 'http',
            '[access_level]': str(config[season]['access_level']),
            '[division]': division,
            '[format]': str(config[season]['format']),
            '[ncaafb_season]': season_type,
            '[version]': str(config[season]['version']),
            '[year]': year,
            '[your_api_key]': str(config[season]['api_key']),
        }

        for placeholder, value in placeholders.items():
            if placeholder in url:
                url = url.replace(placeholder, value)

        return url

    @classmethod
    def send_request(cls, url):
        try:
            response = requests.get(url=url)
            return smart_str(response.text)
        except requests.exceptions.RequestException:
            raise



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.urls import reverse
from django.utils.html import mark_safe
from django.utils.translation import gettext as _


class ReadOnlyAdminMixin(object):
    read_write_fields = []
    explicit_readonly_fields = []

    # def get_readonly_fields(self, request, obj=None):
    #     fields = [f.name for f in self.model._meta.fields if f.name not in self.read_write_fields]
    #     return list(set(fields + list(self.explicit_readonly_fields)))
    #
    # def has_delete_permission(self, request, obj=None):
    #     return False
    # #
    # def has_add_permission(self, request, obj=None):
    #     return False
    # #
    # def get_actions(self, request, obj=None):
    #     actions = super(ReadOnlyAdminMixin, self).get_actions(request)
    #     if 'delete_selected' in actions:
    #         del actions['delete_selected']
    #     return actions


class LinkedRelatedInlineMixin(object):
    """
    This InlineAdmin mixin links the first field to the row object's own admin
    change form.

    NOTE: If the first field is editable, it is undefined what will happen.
    For best results, consider making all fields readonly (since they can be
    edited with ease by following the link), and disabling the ability to add
    new objects by overriding has_add_permission() on the inline to always
    return false.
    """

    extra = 0

    class ReverseLink:

        allow_tags = True

        def __init__(self, display_link="link"):
            self.display_link = display_link
            self.short_description = display_link

        def __call__(self, obj):
            model_name = obj.__class__.__name__.lower()
            admin_link = reverse(
                "admin:{app_label}_{model_name}_change".format(
                    app_label=obj._meta.app_label.lower(),
                    model_name=model_name,
                ), args=(obj.id, ))
            return '<a href="{admin_link}" title="{title}">{link}</a>'.format(
                admin_link=admin_link,
                title=_('Click to view or edit this {0}').format(
                    obj._meta.verbose_name),
                link=getattr(obj, self.display_link))

    def __init__(self, parent_model, admin_site):
        self.original_fields = self.get_fields_list(None)
        if len(self.original_fields):
            self.fields = ["reverse_link", ] + self.original_fields[1:]
        else:
            self.fields = ["reverse_link"]
        self.reverse_link = self.ReverseLink(self.original_fields[0])
        super(LinkedRelatedInlineMixin, self).__init__(parent_model, admin_site)

    def get_fields_list(self, request, obj=None):
        """
        Returns a list of the AdminModel's declared `fields`, or, constructs it
        from the object, then, removes any `exclude`d items.
        """
        if self.fields:
            fields = self.fields
        else:
            fields = [f.name for f in self.model._meta.local_fields]
        if fields and self.exclude:
            fields = [f for f in fields if f not in self.exclude]
        if fields:
            return list(fields)
        else:
            return []

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(
            LinkedRelatedInlineMixin, self).get_readonly_fields(request, obj)
        if "reverse_link" not in readonly_fields:
            readonly_fields = list(readonly_fields) + ["reverse_link", ]
        # We want all fields to be readonly for this inline
        return readonly_fields


class LinkedFieldsMixin(object):
    """
    This mixin will link fields to their respective change-form. This should
    only be used on a ModelAdmin. For similar functionality for Inlines see:
    LinkedRelatedInlineMixin.

    if linked_fields is None, then all FK or O2O fields are used.

    To use, merely add the class property `linked_fields` with a list of FK fields.
    """
    linked_fields = None
    prefix = 'linked_'

    @classmethod
    def add_new_method(cls, model, field):
        """
        Creates a new method on the AdminModel class that returns the field as
        a link to its change form.

        :return: The name of the new method
        """
        method_name = '{0}{1}'.format(cls.prefix, field)
        fld_model = model._meta.get_field(field).remote_field.model.__name__.lower().replace('_', '')
        app_label = model._meta.get_field(field).remote_field.model._meta.app_label

        def new_method(self, obj):
            pattern = 'admin:{0}_{1}_change'.format(app_label, fld_model)
            url = reverse(pattern, args=(getattr(obj, field).pk, ))
            return mark_safe('{value} &nbsp;<a href="{url}" class="inlinechangelink"></a>'.format(url=url, value=getattr(obj, field)))
        new_method.short_description = field

        setattr(cls, method_name, new_method)
        return method_name

    def __init__(self, model, admin_site):
        super(LinkedFieldsMixin, self).__init__(model, admin_site)
        if self.linked_fields is None:
            self.linked_fields = [
                f.name for f in model._meta.get_fields()
                if (f.one_to_one or f.many_to_one) and f.related_model
            ]
        for field in self.linked_fields:
            self.add_new_method(model, field)

    def get_readonly_fields(self, request, obj=None):
        """
        Ensures that the new linked_fields are in readonly_fields.
        """
        readonly_fields = super(LinkedFieldsMixin, self).get_readonly_fields(request, obj=obj) or []
        readonly_fields = list(readonly_fields)
        return readonly_fields + [self.prefix + fld for fld in self.linked_fields]

    @classmethod
    def mapped(cls, mapping, field):
        if not isinstance(field, str) and hasattr(field, '__iter__'):
            # Recurse!
            return [cls.mapped(mapping, fld) for fld in field]
        else:
            if field in mapping:
                return mapping[field]
            else:
                return field

    def get_fieldsets(self, request, obj=None):
        """
        Replaces any occurrence of a field in linked_fields with its
        method counterpart.
        """
        mapping = {fld: self.prefix + fld for fld in self.linked_fields}

        fieldsets = []
        for fieldset in super(LinkedFieldsMixin, self).get_fieldsets(request, obj=obj):
            fields = fieldset[1]['fields']
            fieldset[1]['fields'] = [self.mapped(mapping, f) for f in fields]
            fieldsets.append(fieldset)
        return fieldsets

    @staticmethod
    def uniq(seq):
        """
        Removes duplicates from list whilst preserving order.
        """
        seen = set()
        seen_add = seen.add
        return [x for x in seq if not (x in seen or seen_add(x))]

    def get_fields(self, request, obj=None):
        """
        Replaces any occurrence of a field in linked_fields with its
        method counterpart.
        """
        # TODO: Figure out why we have to de-dupe the list!
        fields = super(LinkedFieldsMixin, self).get_fields(request, obj=obj)
        mapping = {fld: self.prefix + fld for fld in self.linked_fields}
        return self.uniq(self.mapped(mapping, fields))



# Generated by Django 5.0 on 2024-01-13 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0014_season_league'),
    ]

    operations = [
        migrations.AddField(
            model_name='season',
            name='hash',
            field=models.CharField(blank=True, max_length=32, null=True, verbose_name='hash'),
        ),
    ]



from __future__ import unicode_literals

import csv
import random
from os import rename
import importlib



from django.core.management.base import BaseCommand
from django.contrib.auth.models import User, Group
from django.utils.text import slugify
from django.contrib.auth import authenticate

class Command(BaseCommand):
    help = """Loads Participants from a csv file and creates Club based on parsed file name with Participants"""

    def add_arguments(self, parser):
        parser.add_argument('sport', nargs='+', type=str)

    def handle(self, *args, **options):
        sport = options['sport']
        for i in sport:
            mod = (str(i).split(':'))
            package = 'sportspicker-' + mod[0]
            mymod = importlib.import_module(mod[1] + '.utils', package=package)
            print(mymod.import_teams(sport_name=mod[0].title(), league_name=mod[1].title(), verbose=True, force=True))


# Generated by Django 5.0 on 2024-01-14 14:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0016_alter_venue_address_alter_venue_city_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='gameperiod',
            name='sport_radar_period_id',
        ),
        migrations.AddField(
            model_name='gameperiod',
            name='end_date',
            field=models.DateField(blank=True, null=True, verbose_name='end date'),
        ),
        migrations.AddField(
            model_name='gameperiod',
            name='period_id',
            field=models.CharField(blank=True, max_length=40, null=True, verbose_name='game period ID'),
        ),
        migrations.AddField(
            model_name='gameperiod',
            name='start_date',
            field=models.DateField(blank=True, null=True, verbose_name='start date'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from datetime import datetime, timedelta
from time import sleep

from django.core.management.base import BaseCommand
from django.utils import timezone

from sportspicker.models import Club, Contest, LeaderboardPluginModel


class Command(BaseCommand):
    help = """Scores games for a specified Contest"""

    def add_arguments(self, parser):
        parser.add_argument('contest_name', nargs='+', type=str)
        parser.add_argument('api_key', nargs='+', type=str)
        parser.add_argument('--recache', dest='recache', action='store_true')

    def handle(self, *args, **options):
        contest_name = options['contest_name'][0]
        api_key = options['api_key'][0]
        recache = options['recache'] == True
        contest = Contest.objects.get(slug=contest_name)
        cutoff = timezone.now() - timedelta(minutes=200)
        for period in contest.contest_periods.filter(start_date__lte=cutoff):
            for game in period.games.filter(scheduled__lte=cutoff):
                if not game.scored:
                    print game.pull_score(api_key)
                if game.scored and recache:
                    self.re_cache_leaderboards(game)
                sleep(1)

    def re_cache_leaderboards(self, game):
        """
        Recache all the leaderboards for this game.
        """
        for club in Club.objects.filter(contests__games=game):
            club_ids = [club.pk, ]
            for contest in club.contests.filter(games=game):
                for contest_period in contest.contest_periods.filter(games=game):
                    LeaderboardPluginModel.get_leaderboard(club_ids, scope=contest_period, recache=True)
                LeaderboardPluginModel.get_leaderboard(club_ids, scope=contest, recache=True)
            LeaderboardPluginModel.get_leaderboard(club_ids, scope=club, recache=True)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from num2words import num2words

from django import template

from classytags.arguments import Argument
from classytags.core import Options
from classytags.helpers import AsTag

register = template.Library()


@register.filter
def ordinal_word(value):
    """Converts 1 => first, 2 => second, 3 => third."""
    try:
        value = int(value)
        return num2words(value, ordinal=True)
    except ValueError:
        return value



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db.models import F, Q
from django.http import HttpResponseRedirect
from django.utils.decorators import method_decorator
from django.utils.functional import cached_property
from django.views.generic import DetailView, ListView, UpdateView

from ..models import Conference, Game, Team

from .decorators import sports_admin_required
from .forms import GameForm


class SportAdminMixin(object):
    @method_decorator(sports_admin_required())
    def dispatch(self, request, *args, **kwargs):
        return super(SportAdminMixin, self).dispatch(request, *args, **kwargs)


class ConferenceMixin(object):
    @cached_property
    def conference(self):
        return Conference.objects.filter(pk=self.kwargs.get('conference_pk', -1)).first()

    def get_context_data(self, **kwargs):
        context = super(ConferenceMixin, self).get_context_data(**kwargs)
        context['conference'] = self.conference
        return context


class ConferenceListView(SportAdminMixin, ListView):
    model = Conference
    http_method_names = ['get', ]

    def get_queryset(self):
        qs = super(ConferenceListView, self).get_queryset()
        return qs.order_by('name')


class ConferenceDetailView(SportAdminMixin, DetailView):
    model = Conference
    pk_url_kwarg = 'conference_pk'


class TeamDetailView(SportAdminMixin, ConferenceMixin, DetailView):
    model = Team
    pk_url_kwarg = 'team_pk'

    def get_object(self, queryset=None):
        return super(TeamDetailView, self).get_object(queryset=queryset)

    def get_context_data(self, **kwargs):
        context = super(TeamDetailView, self).get_context_data(**kwargs)
        team = self.get_object()
        context['games'] = (Game.objects.filter(Q(home_id=team.pk) | Q(away_id=team.pk))
                                        .annotate(season=F('game_period__season__label'))
                                        .order_by('scheduled'))
        return context


class GameDetailView(SportAdminMixin, UpdateView):
    model = Game
    form_class = GameForm
    pk_url_kwarg = 'game_pk'
    template_name = 'sports/game_detail.html'
    # This will store the value of 'next', if provided
    redirect_url = None

    def form_valid(self, form):
        """
        If the form is valid, save the associated model.
        """
        self.object = form.save()
        redirect_url = form.cleaned_data['redirect_url']
        return HttpResponseRedirect(redirect_url or self.get_success_url())

    def get_context_data(self, **kwargs):
        context = super(GameDetailView, self).get_context_data(**kwargs)
        return context

    def get_form_kwargs(self):
        """
        Returns the keyword arguments for instantiating the form.
        """
        kwargs = super(GameDetailView, self).get_form_kwargs()
        kwargs.update({'redirect_url': self.redirect_url})
        return kwargs

    def get(self, request, *args, **kwargs):
        """
        If a `next` is set, ensure its added to the form.
        """
        self.redirect_url = request.GET.get('next', None)
        return super(GameDetailView, self).get(request, *args, **kwargs)



# Generated by Django 5.0 on 2024-01-09 22:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0009_player_player_id'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='player',
            options={'ordering': ('team', 'last_name', 'first_name'), 'verbose_name': 'Player', 'verbose_name_plural': 'Players'},
        ),
        migrations.AddField(
            model_name='player',
            name='birth_date',
            field=models.DateField(blank=True, null=True, verbose_name='birth date'),
        ),
        migrations.AddField(
            model_name='player',
            name='height',
            field=models.CharField(blank=True, max_length=10, verbose_name='height'),
        ),
        migrations.AddField(
            model_name='player',
            name='jersey_number',
            field=models.CharField(blank=True, max_length=10, verbose_name='jersey number'),
        ),
        migrations.AddField(
            model_name='player',
            name='position',
            field=models.CharField(blank=True, max_length=10, verbose_name='position'),
        ),
        migrations.AddField(
            model_name='player',
            name='primary_position',
            field=models.CharField(blank=True, max_length=10, verbose_name='primary position'),
        ),
        migrations.AddField(
            model_name='player',
            name='weight',
            field=models.CharField(blank=True, max_length=10, verbose_name='weight'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import json
from argparse import Namespace
from django.core.management.base import BaseCommand

from ...models import Conference, Division, Team, Venue


class Command(BaseCommand):
    help = """Loads NFL team data from JSON."""

    def add_arguments(self, parser):
        parser.add_argument('file_name', nargs='+', type=str)

    @classmethod
    def process_teams(cls, team_list, conference, division=None):
        """
        Creates teams and their venues.
        """
        for team in team_list:
            # Venue
            if getattr(team, 'venue', False):
                venue_obj, created = Venue.objects.update_or_create(
                    venue_id=team.venue.id, defaults={
                        'name': getattr(team.venue, 'name', ''),
                        'country': getattr(team.venue, 'country', ''),
                        'city': getattr(team.venue, 'city', ''),
                        'state': getattr(team.venue, 'state', ''),
                        'zip_code': getattr(team.venue, 'zip_code', ''),
                    }
                )
                print('{0} venue: {1}'.format('Created' if created else 'Updated', venue_obj))
            else:
                venue_obj = None

            # Team
            team_obj, created = Team.objects.update_or_create(
                team_id=team.id, defaults={
                    'name': team.name,
                    'market': team.market,
                    'conference': conference,
                    'division': division,
                    'venue': venue_obj,
                }
            )
            print('{0} team: {1}'.format('Created' if created else 'Updated', team_obj))

    @classmethod
    def process_conferences(cls, conference_list):
        """
        Creates conferences and divisions (if any) as well as teams and venues
        (via process_teams)
        """
        for conference in conference_list:
            # Conference
            conference_obj, created = Conference.objects.update_or_create(
                conference_id=conference.id, defaults={
                    'name': conference.name
                }
            )
            print('{0} conference: {1}'.format('Created' if created else 'Updated', conference_obj))

            # print(conference.subdivisions[0].id)
            if getattr(conference, 'divisions', False):
                for division in conference.divisions:
                    # Division
                    division_obj, created = Division.objects.update_or_create(
                        division_id=division.id, defaults={
                            'name': division.name,
                            'conference': conference_obj,
                        }
                    )
                    print('{0} division: {1}'.format('Created' if created else 'Updated', division_obj))
                    cls.process_teams(division.teams, conference_obj, division_obj)
            elif getattr(conference, 'teams', None):
                cls.process_teams(conference.teams, conference_obj)
            else:
                pass

    def handle(self, *args, **options):
        file_name = options['file_name'][0]
        file_json = open(file_name).read()
        data = json.loads(file_json, object_hook=lambda d: Namespace(**d))

        self.process_conferences(data.conferences)



# Generated by Django 5.0 on 2024-01-03 15:28

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0005_alter_sport_options_league'),
    ]

    operations = [
        migrations.AddField(
            model_name='conference',
            name='league',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='conferences', to='sports.league', verbose_name='League'),
        ),
    ]



# Generated by Django 5.0 on 2024-01-03 14:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0004_sport'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='sport',
            options={'verbose_name': 'Sport', 'verbose_name_plural': 'Sports'},
        ),
        migrations.CreateModel(
            name='League',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=40, verbose_name='League Name')),
                ('sport', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leagues', to='sports.sport', verbose_name='Sport')),
            ],
            options={
                'verbose_name': 'League',
                'verbose_name_plural': 'Leagues',
            },
        ),
    ]



# Generated by Django 5.0 on 2024-01-10 20:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0012_season_season_id_alter_season_label'),
    ]

    operations = [
        migrations.RenameField(
            model_name='season',
            old_name='season_name',
            new_name='name',
        ),
        migrations.RenameField(
            model_name='season',
            old_name='season_type',
            new_name='type',
        ),
        migrations.AddField(
            model_name='season',
            name='end_date',
            field=models.DateField(blank=True, null=True, verbose_name='end date'),
        ),
        migrations.AddField(
            model_name='season',
            name='start_date',
            field=models.DateField(blank=True, null=True, verbose_name='start date'),
        ),
        migrations.AddField(
            model_name='season',
            name='year',
            field=models.CharField(default='', max_length=4, verbose_name='year'),
        ),
    ]



# Generated by Django 5.0 on 2024-01-03 20:54

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0007_alter_conference_league'),
    ]

    operations = [
        migrations.CreateModel(
            name='Player',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=255, verbose_name='first name')),
                ('last_name', models.CharField(max_length=255, verbose_name='last name')),
                ('team', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sports.team')),
            ],
        ),
    ]



# Generated by Django 5.0 on 2024-01-10 19:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0011_alter_player_height_alter_player_jersey_number_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='season',
            name='season_id',
            field=models.CharField(default=1, max_length=40, verbose_name='season id'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='season',
            name='label',
            field=models.CharField(blank=True, default=None, max_length=100, null=True, verbose_name='label'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.management.base import BaseCommand
from django.contrib.auth import authenticate
from .import_club import Command as ClubCommand

from sportspicker.models import Club, ClubUser


class Command(BaseCommand):
    help = """Tests each user in a Club with the expected, initial generated username and password"""

    def add_arguments(self, parser):
        parser.add_argument('club_name', nargs='+', type=str)

    def handle(self, *args, **options):
        club_name = options['club_name'][0]
        club = Club.objects.get(slug=club_name)

        for x in ClubUser.objects.filter(club=club):
            generated_password = ClubCommand.generate_password(
                fname=x.user.first_name, lname=x.user.last_name, cname=club.name, method='last_name'
            )
            result = authenticate(username=x.user.username, password=generated_password)
            if result is not None:
                print 'User ' + str(x.user.username) + ' successfully logged in with password ' + generated_password
            else:
                print 'Error: User ' + str(x.user.username) + ' failed to log in with password ' + generated_password


# Generated by Django 5.0 on 2024-01-10 20:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0013_rename_season_name_season_name_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='season',
            name='league',
            field=models.ForeignKey(default=3, on_delete=django.db.models.deletion.CASCADE, related_name='seasons', to='sports.league', verbose_name='League'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import user_passes_test


def sports_admin_required(function=None, redirect_field_name=REDIRECT_FIELD_NAME, login_url=None):
    """
    Allows access to the decorated View if user `is_sportsadmin` or `is_staff`
    """
    actual_decorator = user_passes_test(
        lambda u: u.user_data.is_sportsadmin or u.is_staff,
        login_url=login_url,
        redirect_field_name=redirect_field_name
    )
    if function:
        return actual_decorator(function)
    return actual_decorator



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from random import choice
from datetime import datetime

from django.core.management.base import BaseCommand
from django.utils import timezone
from django.contrib.auth import authenticate
from .import_club import Command as ClubCommand

from sportspicker.models import Club, Contest, ClubUser, GamePick


class Command(BaseCommand):
    help = """Simulates a season of picks for a specified club"""

    def add_arguments(self, parser):
        parser.add_argument('club_name', nargs='+', type=str)
        parser.add_argument('contest_name', nargs='+', type=str)

    def handle(self, *args, **options):
        club_name = options['club_name'][0]
        club = Club.objects.get(slug=club_name)
        contest_name = options['contest_name'][0]
        contest = Contest.objects.get(slug=contest_name)

        for x in ClubUser.objects.filter(club=club):

            for pindex, period in enumerate(contest.contest_periods.all()):

                for __, game in enumerate(period.games.all()):

                    choices = (game.away, game.home)
                    if game.pk == period.tie_breaker_game_id:
                        homescore = choice(range(0, 50))
                        visitorscore = choice(range(0, 50))
                    else:
                        homescore = None
                        visitorscore = None
                    guess, created = GamePick.objects.update_or_create(
                        club_user_id=x.pk,
                        game_id=game.pk,
                        defaults={
                            'winner': choice(choices),
                            'home_score': homescore,
                            'visitor_score': visitorscore,
                            'pick_time': timezone.now()
                        }
                    )
                    print guess



# Generated by Django 5.0 on 2024-01-03 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0002_team_alias'),
    ]

    operations = [
        migrations.AddField(
            model_name='venue',
            name='address',
            field=models.CharField(blank=True, default='', max_length=128, verbose_name='address'),
        ),
    ]



# Generated by Django 5.0 on 2024-01-03 14:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0003_venue_address'),
    ]

    operations = [
        migrations.CreateModel(
            name='Sport',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=40, verbose_name='Sport Name')),
            ],
        ),
    ]



# Generated by Django 5.0 on 2024-01-14 16:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0017_remove_gameperiod_sport_radar_period_id_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='gameperiod',
            name='label',
            field=models.CharField(blank=True, default=None, max_length=100, null=True, verbose_name='label'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import csv
import random
from os import rename

from django.core.management.base import BaseCommand
from django.contrib.auth.models import User, Group
from django.utils.text import slugify
from django.contrib.auth import authenticate

from sportspicker.models import Club, ClubUser, UserProfile


class Command(BaseCommand):
    help = """Loads Participants from a csv file and creates Club based on parsed file name with Participants"""

    def add_arguments(self, parser):
        parser.add_argument('file_name', nargs='+', type=str)
        parser.add_argument('password_method', nargs='+', type=str)
        parser.add_argument('force_username_change', nargs='+', type=str)
        parser.add_argument('force_password_change', nargs='+', type=str)

    def handle(self, *args, **options):
        file_name = options['file_name'][0]
        csv_write_file_name = (file_name + '.tmp')
        password_method = unicode(options['password_method'][0]).lower()
        force_username_change = str(options['force_username_change'][0]).lower()
        force_password_change = str(options['force_password_change'][0]).lower()
        clubname = self.derive_label_from_file_name(file_name)
        club = (self.add_club_to_contest(clubname))
        if Group.objects.filter(name='Picker').exists() is False:
            group = Group.objects.get_or_create(name='Picker')
            # Code to add permission to group ???
        else:
            group = Group.objects.get(name='Picker')

        csvfieldnames = (
                'ID', 'PSWD', 'FNAME', 'LNAME', 'EXPERT', 'DISABLE', 'ADDRESS', 'CITY', 'STATE', 'ZIP', 'COMPANY',
            )

        with open(file_name) as csvfile:
            reader = csv.DictReader(csvfile)
            csvwritefile = open(csv_write_file_name, 'w')
            writer = csv.DictWriter(csvwritefile, fieldnames=csvfieldnames)
            writer.writeheader()
            for row in reader:
                # this is temporary until we get an export with email addresses
                # ToDo - Split this off into its own method
                expertstring = ''
                try:
                    expertvalue = row['EXPERT'].strip()
                except:
                    expertvalue = 'N'
                if force_username_change == 'yes':
                    username = self.generate_username((row['FNAME']), (row['LNAME']), clubname)
                elif (row['ID']) != "" and (row['ID']) is not None:
                    username = (row['ID']).strip()
                else:
                    username = self.generate_username((row['FNAME']), (row['LNAME']), clubname)
                if force_password_change == 'yes':
                    password = self.generate_password((row['FNAME']), (row['LNAME']), clubname, method=password_method)
                elif (row['PSWD']) != "" and (row['PSWD']) is not None:
                    password = (row['PSWD'])
                else:
                    password = self.generate_password((row['FNAME']), (row['LNAME']), clubname, method=password_method)
                try:
                    if expertvalue == 'Y':
                        is_expert = True
                        expertstring = ' (Expert)'
                    else:
                        is_expert = False
                except:
                    is_expert = False

                try:
                    if row['NEWSLETTER'].strip() == 'Y':
                        gets_newsletter = True
                    else:
                        gets_newsletter = False
                except:
                    gets_newsletter = False

                try:
                    if row['REP'].strip() == 'Y':
                        is_rep = True
                        expertstring += ' (Player Rep)'
                    else:
                        is_rep = False
                except:
                    is_rep = False

                if not User.objects.filter(username=username).exists():
                    user = User.objects.create_user(username=username, password=password)
                    user.first_name = (row['FNAME']).strip()
                    user.last_name = (row['LNAME']).strip()
                    user.save()
                    user.groups.add(group)
                    clubuser = club.add_member(user, is_rep=is_rep)

                    clubuser.is_rep = is_rep
                    clubuser.is_expert = is_expert
                    clubuser.gets_newsletter = gets_newsletter
                    clubuser.save()

                    user_profile, _ = UserProfile.objects.update_or_create(user_id=user.id, defaults={
                        'address': row['ADDRESS'].strip(),
                        'city': row['CITY'].strip(),
                        'state': row['STATE'].strip(),
                        'zip_code': row['ZIP'].strip(),
                        'company': row['COMPANY'].strip(),
                    })

                    writer.writerow(
                        {
                            'ID': user.username,
                            'PSWD': password,
                            'FNAME': user.first_name,
                            'LNAME': user.last_name,
                            'EXPERT': expertvalue,
                            'DISABLE': 'N',
                            'ADDRESS': user_profile.address,
                            'CITY': user_profile.city,
                            'STATE': user_profile.state,
                            'ZIP': user_profile.zip_code,
                            'COMPANY': user_profile.company,
                        }
                    )
                    print("Added {first_name} {last_name}{expertstring} to {club_name}".format(
                        first_name=user.first_name,
                        last_name=user.last_name,
                        club_name=club.name,
                        expertstring=expertstring
                        ))
                else:

                    user = User.objects.get(username=username)
                    if user.groups.filter(name=group.name).exists() is False:
                        user.groups.add(group)
                    if authenticate(username=username, password=password) is None:
                        user.set_password(password)
                        print("Password for {first_name} {last_name} updated to {password}".format(
                            first_name=user.first_name,
                            last_name=user.last_name,
                            password=password
                        ))
                    clubuser = ClubUser.objects.get(user=user, club=club)
                    user_profile, _ = UserProfile.objects.get_or_create(user_id=user.id, defaults={
                        'address': row['ADDRESS'],
                        'city': row['CITY'],
                        'state': row['STATE'],
                        'zip_code': row['ZIP'],
                        'company': row['COMPANY'],
                    })
                    if is_rep != clubuser.is_rep:
                        clubuser.is_rep = is_rep
                    if is_expert != clubuser.is_expert:
                        clubuser.is_expert = is_expert

                    clubuser.save_from_command_line()

                    writer.writerow(
                        {
                            'ID': user.username,
                            'PSWD': password,
                            'FNAME': user.first_name,
                            'LNAME': user.last_name,
                            'EXPERT': expertvalue,
                            'DISABLE': 'N',
                            'ADDRESS': clubuser.address,
                            'CITY': clubuser.city,
                            'STATE': clubuser.state,
                            'ZIP': clubuser.zip_code,
                            'COMPANY': clubuser.company,
                        }
                    )

                    print("Duplicate {first_name} {last_name}{expertstring} already exists in {club_name}".format(
                        first_name=row['FNAME'],
                        last_name=row['LNAME'],
                        club_name=club.name,
                        expertstring=expertstring
                    ))

        rename(file_name, file_name + '.bak')
        rename(csv_write_file_name, file_name)

    @classmethod
    def derive_label_from_file_name(cls, file_name):
        try:
            file_name = (((file_name.split('/'))[-1]).split('.')[0]).replace('-', ' ')
            file_name = file_name.replace(' games', '')
            return file_name
        except:
            return file_name

    @classmethod
    def add_club_to_contest(cls, name):
        club, created = Club.objects.get_or_create(
            name=name, slug=(slugify(name))
        )
        return club

    @classmethod
    def generate_username(cls, fname, lname, cname):
        out = str(fname).strip() + str(lname).strip()
        out = out.replace('.', '')
        out += '@' + cname + '.club'
        out = out.replace(' ', '')
        out = out.lower()
        return out

    @classmethod
    def generate_password(cls, fname, lname, cname, method='random_int'):

        def random_int(lower=10000, upper=99999):
            out = str(random.randint(lower, upper))
            return out

        def last_name(lname):
            out = str(lname).strip()
            out = out.replace(' ', '')
            out = out.replace('.', '')
            out = out.replace(',', '')
            out = out.lower()
            return out

        if method == 'random_int':
            return random_int()

        elif method == 'last_name':
            return last_name(lname)



# Generated by Django 4.0.3 on 2022-04-07 03:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Conference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('conference_id', models.CharField(default='', max_length=40, verbose_name='conference ID')),
                ('name', models.CharField(default='', max_length=64, verbose_name='name')),
            ],
            options={
                'verbose_name': 'conference',
                'verbose_name_plural': 'conferences',
            },
        ),
        migrations.CreateModel(
            name='Division',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('division_id', models.CharField(default='', max_length=40, verbose_name='division ID')),
                ('name', models.CharField(default='', max_length=32, verbose_name='name')),
                ('conference', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='divisions', to='sports.conference', verbose_name='conference')),
            ],
            options={
                'verbose_name': 'division',
                'verbose_name_plural': 'divisions',
            },
        ),
        migrations.CreateModel(
            name='Game',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('game_id', models.CharField(default='', max_length=40, verbose_name='game ID')),
                ('scheduled', models.DateTimeField(verbose_name='scheduled')),
                ('scored', models.DateTimeField(blank=True, null=True, verbose_name='scored')),
                ('summary', models.TextField(blank=True, null=True, verbose_name='summary')),
                ('coverage', models.CharField(blank=True, default='', max_length=32, verbose_name='coverage')),
                ('home_rotation', models.CharField(blank=True, default='', max_length=32, verbose_name='home rotation')),
                ('away_rotation', models.CharField(blank=True, default='', max_length=32, verbose_name='away rotation')),
                ('status', models.CharField(blank=True, default='', max_length=32, verbose_name='status')),
                ('home_points', models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='home points')),
                ('away_points', models.PositiveSmallIntegerField(blank=True, null=True, verbose_name='away points')),
            ],
            options={
                'verbose_name': 'game',
                'verbose_name_plural': 'games',
                'ordering': ['scheduled'],
            },
        ),
        migrations.CreateModel(
            name='Season',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(default='', max_length=100, verbose_name='label')),
                ('season_name', models.CharField(default='', max_length=32, verbose_name='name')),
                ('season_type', models.CharField(default='', max_length=32, verbose_name='type')),
            ],
            options={
                'verbose_name': 'season',
                'verbose_name_plural': 'seasons',
            },
        ),
        migrations.CreateModel(
            name='Venue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('venue_id', models.CharField(default='', max_length=40, verbose_name='venue ID')),
                ('name', models.CharField(default='', max_length=128, verbose_name='name')),
                ('country', models.CharField(blank=True, default='', max_length=32, verbose_name='country')),
                ('city', models.CharField(blank=True, default='', max_length=32, verbose_name='city')),
                ('state', models.CharField(blank=True, default='', max_length=32, verbose_name='state')),
                ('zip_code', models.CharField(blank=True, default='', max_length=10, verbose_name='ZIP Code')),
            ],
            options={
                'verbose_name': 'venue',
                'verbose_name_plural': 'venues',
            },
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('team_id', models.CharField(default='', max_length=40, verbose_name='team ID')),
                ('name', models.CharField(blank=True, default='', max_length=64, verbose_name='name')),
                ('market', models.CharField(blank=True, default='', max_length=64, verbose_name='market')),
                ('conference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='teams', to='sports.conference', verbose_name='conference')),
                ('division', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='teams', to='sports.division', verbose_name='division')),
                ('venue', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='teams', to='sports.venue', verbose_name='venue')),
            ],
            options={
                'verbose_name': 'team',
                'verbose_name_plural': 'teams',
                'ordering': ('name', 'division', 'conference'),
            },
        ),
        migrations.CreateModel(
            name='GameSummary',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('home_points', models.PositiveSmallIntegerField(default=0, verbose_name='home points')),
                ('away_points', models.PositiveSmallIntegerField(default=0, verbose_name='away points')),
                ('game', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='game', to='sports.game')),
            ],
            options={
                'verbose_name': 'summary',
                'verbose_name_plural': 'summaries',
            },
        ),
        migrations.CreateModel(
            name='GamePeriod',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sport_radar_period_id', models.CharField(default='', max_length=40, verbose_name='game period ID')),
                ('number', models.PositiveSmallIntegerField(default=0, verbose_name='number')),
                ('season', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='game_periods', to='sports.season', verbose_name='season')),
            ],
            options={
                'verbose_name': 'game period',
                'verbose_name_plural': 'game periods',
                'ordering': ('season', 'number'),
            },
        ),
        migrations.AddField(
            model_name='game',
            name='away',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='game_away', to='sports.team'),
        ),
        migrations.AddField(
            model_name='game',
            name='game_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='games', to='sports.gameperiod', verbose_name='game period'),
        ),
        migrations.AddField(
            model_name='game',
            name='home',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='game_home', to='sports.team'),
        ),
        migrations.AddField(
            model_name='game',
            name='venue',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='games', to='sports.venue', verbose_name='venue'),
        ),
        migrations.CreateModel(
            name='Broadcast',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('broadcast_type', models.CharField(default='', max_length=40, verbose_name='type')),
                ('broadcast_address', models.CharField(default='', max_length=64, verbose_name='address')),
                ('game', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sports.game', verbose_name='game')),
            ],
            options={
                'verbose_name': 'broadcast',
                'verbose_name_plural': 'broadcasts',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import forms
from django.contrib import admin
from django.db.models import Count
from django.utils.translation import gettext as _

from django_select2.forms import Select2Widget

from ..models import Sport, League, Broadcast, Conference, Division, Game, GamePeriod, Season, Team, Player, Venue, GameSummary

from .mixins import ReadOnlyAdminMixin, LinkedFieldsMixin


class BroadcastInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Broadcast
    fields = ('broadcast_type', 'broadcast_address', )
    extra = 0
    show_change_link = True


class DivisionInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Division
    fields = ('name', )
    extra = 0
    readonly_fields = ('name', )
    show_change_link = True


class GameInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Game
    fields = ('scheduled', 'home', 'away', )
    extra = 0
    show_change_link = True


# TODO: Currently, we need to use two "GameInlines" when used on Team. This is
# because Game has two FKs (soon to be 3) that point to Team. At some point,
# we can create a new, custom inline that will re-unify 'home' and 'away' into
# a single inline. Or at least re-label these two so they communicate better.
class HomeGameInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Game
    fk_name = 'home'
    fields = ('scheduled', 'away', )
    extra = 0
    show_change_link = True


class AwayGameInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Game
    fk_name = 'away'
    fields = ('scheduled', 'home', )
    extra = 0
    show_change_link = True


class GamePeriodInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = GamePeriod
    extra = 0
    fields = ('season', 'number', )
    show_change_link = True

    def get_queryset(self, request):
        qs = super(GamePeriodInline, self).get_queryset(request)
        return qs.annotate(num_games=Count('games'))


class TeamInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Team
    extra = 0
    fields = ('name', 'market', 'conference', 'division', )
    readonly_fields = ('name', 'market', 'conference', 'division', )
    show_change_link = True

class LeagueInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = League
    extra = 0
    fields = ('name',)
    readonly_fields = ('name',)
    show_change_link = False

class PlayersInline(ReadOnlyAdminMixin, admin.TabularInline):
    model = Player
    extra = 0
    fields = ('first_name', 'last_name',)
    readonly_fields = ('first_name', 'last_name',)
    show_change_link = False


# ========== INLINES ^ / ADMINS v ==============

class SportAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    inlines = (LeagueInline,)

admin.site.register(Sport, SportAdmin)

class LeagueAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    # inlines = (DivisionInline,)
    pass

admin.site.register(League, LeagueAdmin)


class BroadcastAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('game', 'broadcast_type', 'broadcast_address', )
    list_filter = ('broadcast_type', )
    fields = ('game', 'broadcast_type', 'broadcast_address', )

admin.site.register(Broadcast, BroadcastAdmin)

class GameSummaryAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('game', 'away_points', 'home_points', )
    fields = ('game', 'away_points', 'home_points', )

admin.site.register(GameSummary, GameSummaryAdmin)


class ConferenceAdmin(ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('name', 'num_teams', )
    readonly_fields = ('name', 'league', )
    inlines = (DivisionInline, TeamInline, )
    fieldsets = (
        (None, {
            'fields': (
                'name',
                'league',
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse', ),
            'fields': (
                'conference_id',
            )
        })
    )

    def get_queryset(self, request):
        qs = super(ConferenceAdmin, self).get_queryset(request)
        return qs.annotate(num_teams=Count('teams'))

    def num_teams(self, obj):
        return obj.num_teams
    num_teams.admin_order_field = 'num_teams'
    num_teams.short_description = '# Teams'

admin.site.register(Conference, ConferenceAdmin)


class DivisionAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('name', 'conference', 'get_league', 'get_sport', 'num_teams', )
    list_display_links = ('name', )
    inlines = (TeamInline, )
    fieldsets = (
        (None, {
            'fields': (
                'name', 'conference',
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse', ),
            'fields': (
                'division_id',
            )
        })
    )

    def get_queryset(self, request):
        qs = super(DivisionAdmin, self).get_queryset(request)
        return qs.annotate(num_teams=Count('teams'))

    def num_teams(self, obj):
        return obj.num_teams
    num_teams.admin_order_field = 'num_teams'
    num_teams.short_description = '# Teams'
    def get_league(self, obj):
        return obj.conference.league
    get_league.admin_order_field = 'num_teams'
    get_league.short_description = 'League'
    def get_sport(self, obj):
        return obj.conference.league.sport

    get_sport.admin_order_field = 'get_sport'
    get_sport.short_description = 'Sport'

admin.site.register(Division, DivisionAdmin)


class GameAdminForm(forms.ModelForm):
    # home = ModelSelect2Field(queryset=Team.objects.all())
    # away = ModelSelect2Field(queryset=Team.objects.all())
    # venue = ModelSelect2Field(queryset=Venue.objects.all())

    class Meta:
        model = Game
        fields = '__all__'
        widgets = {
            'home': Select2Widget,
            'away': Select2Widget,
            'venue': Select2Widget,
        }

    class Media:
        js = (
            'https://ajax.googleapis.com/ajax/libs/jquery/2.2.4/jquery.min.js',
        )


# LinkedFieldsMixin,
class GameAdmin(admin.ModelAdmin):
    list_display = ('gamesummary', 'scheduled', 'game_period', 'away', 'away_points', 'home', 'home_points', 'venue', )
    inlines = (BroadcastInline, )
    list_filter = ('away', 'scheduled', 'scored', )
    form = GameAdminForm
    ordering = ['-scheduled',]

    def gamesummary(self, obj):
        if obj.scored is not None:
            return '{o.away} {o.away_points}, {o.home.full_name} {o.home_points}'.format(o=obj)
        else:
            return '{o.away} at {o.home.full_name}'.format(o=obj)
    gamesummary.short_description = 'Summary'

    def gameperiod(self, obj):
        return '{o.game_period_name}'.format(o=obj)
    gameperiod.short_description = 'Period'

    def season(self, obj):
        return '{o.season_name}'.format(o=obj)
    season.short_description = 'Year'

admin.site.register(Game, GameAdmin)


# LinkedFieldsMixin, ReadOnlyAdminMixin,
# explicit_readonly_fields = ('earliest_game_start', 'latest_game_start', )
class GamePeriodAdmin(admin.ModelAdmin):
    readonly_fields = ('earliest_game_start', 'latest_game_start', )
    inlines = (GameInline, )
    list_display = ('label', 'season','get_league','get_sport', 'number', 'start_date', 'end_date', 'num_games', )
    read_write_fields = ['season', ]
    ordering = ['-start_date', 'number']

    fieldsets = (
        (None, {
            'fields': (
                'number',
                'season',
                ('earliest_game_start', 'latest_game_start', ),
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse', ),
            'fields': (
                'period_id',
            )
        })
    )

    def get_queryset(self, request):
        qs = super(GamePeriodAdmin, self).get_queryset(request)
        return qs.annotate(num_games=Count('games'))

    def num_games(self, obj=None):
        return obj.num_games
    num_games.short_description = '# games'
    num_games.admin_order_field = 'num_games'

    def get_league(self, obj):
        return obj.season.league

    get_league.admin_order_field = 'num_teams'
    get_league.short_description = 'League'

    def get_sport(self, obj):
        return obj.season.league.sport

    get_sport.admin_order_field = 'get_sport'
    get_sport.short_description = 'Sport'

admin.site.register(GamePeriod, GamePeriodAdmin)


class SeasonAdmin(admin.ModelAdmin):
    list_display = ('label', 'get_sport', 'league', 'name', 'type', 'start_date', 'end_date', 'hash' )
    readonly_fields = ('name', 'type', )
    inlines = (GamePeriodInline, )

    def get_sport(self, obj):
        return obj.league.sport

    get_sport.admin_order_field = 'get_sport'
    get_sport.short_description = 'Sport'

admin.site.register(Season, SeasonAdmin)


class TeamAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('name', 'market', 'conference', 'division', 'venue', )
    list_filter = ('conference', )
    search_fields = ('name', 'market')
    inlines = (HomeGameInline, AwayGameInline, PlayersInline,  )
    fieldsets = (
        (None, {
            'fields': (
                'name', 'alias', 'market', 'conference', 'division', 'venue',
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse', ),
            'fields': (
                'team_id',
            )
        })
    )

admin.site.register(Team, TeamAdmin)

class PlayerAdmin(LinkedFieldsMixin, ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('last_name', 'first_name',  'team','jersey_number','primary_position')
    list_filter = ('team',)

    fieldsets = (
        (None, {
            'fields': (
                'last_name', 'first_name', 'team','jersey_number','height','weight','position','primary_position','birth_date',
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse',),
            'fields': (
                'player_id',
            )
        })
    )

admin.site.register(Player, PlayerAdmin)


class VenueAdmin(ReadOnlyAdminMixin, admin.ModelAdmin):
    list_display = ('name', 'country', 'city', 'state', 'zip_code', )
    inlines = (TeamInline, GameInline, )
    fieldsets = (
        (None, {
            'fields': (
                'name', 'country', ('address', 'city', 'state', 'zip_code', ),
            )
        }),
        (_('Advanced'), {
            'classes': ('collapse', ),
            'fields': (
                'venue_id',
            )
        })
    )

admin.site.register(Venue, VenueAdmin)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import forms

from ..models import Game


class ReadOnlyFieldsMixin(object):
    # TODO: This should be updated when the project upgrades to Django 1.9+.
    # See: https://docs.djangoproject.com/en/1.9/ref/forms/fields/#disabled
    readonly_fields = []

    def __init__(self, *args, **kwargs):
        """
        Sets the widgets for the read-only fields to be readonly.
        """
        super(ReadOnlyFieldsMixin, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.pk:
            for field in self.readonly_fields:
                self.fields[field].widget.attrs['readonly'] = 'readonly'

    def clean(self):
        """
        Ensures that no form-hacking had occurred for the read-only fields
        """
        cleaned_data = super(ReadOnlyFieldsMixin, self).clean()
        instance = getattr(self, 'instance', None)
        if instance and instance.pk:
            for field in self.readonly_fields:
                cleaned_data[field] = getattr(instance, field, None)
        return cleaned_data


class GameForm(ReadOnlyFieldsMixin, forms.ModelForm):
    readonly_fields = ['game_id', 'coverage', 'status', ]

    redirect_url = forms.CharField(max_length=4096, widget=forms.HiddenInput(), required=False)

    def __init__(self, *args, **kwargs):
        redirect_url = kwargs.pop('redirect_url', None)
        super(GameForm, self).__init__(*args, **kwargs)
        self.fields['redirect_url'].initial = redirect_url

    class Meta:
        model = Game
        fields = [
            'redirect_url',
            'game_id',
            'game_period',
            'scheduled', 'coverage', 'status',
            'away',
            'home',
            'away_points',
            'home_points',
            'venue',
        ]



from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()

with open('LICENSE.rst') as f:
    license = f.read()

setup(
    name='sportspicker',
    version='0.0.3',
    description='Update Me',
    long_description=readme,
    author='Peter D Bethke',
    author_email='pdbethke@siteshell.net',
    url='https://github.com/pdbethke/sportspicker',
    license=license,
    packages=find_packages(),
    include_package_data=True,
    package_dir={'sportspicker': 'sportspicker'},
    install_requires=[
        'django',
        'num2words',
        'django-solo',
        'requests',
        'django-select2',
        'django-classy-tags',
        'sportradar',
    ],
)



# Generated by Django 5.0 on 2024-01-03 21:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sports', '0008_player'),
    ]

    operations = [
        migrations.AddField(
            model_name='player',
            name='player_id',
            field=models.CharField(default=123, max_length=40),
            preserve_default=False,
        ),
    ]



# # -*- coding: utf-8 -*-
#
# from __future__ import unicode_literals
#
# from django.conf.urls import url
#
# from .views import (
#     ConferenceListView,
#     ConferenceDetailView,
#     GameDetailView,
#     TeamDetailView,
# )
#
#
# urlpatterns = [
#     url(r'^$',
#         ConferenceListView.as_view(), name='conference_list'),
#
#     url(r'^(?P<conference_pk>\d+)/$',
#         ConferenceDetailView.as_view(), name='conference_detail'),
#
#     url(r'^(?P<conference_pk>\d+)/team-(?P<team_pk>[-\w]+)/$',
#         TeamDetailView.as_view(), name='team_detail'),
#
#     url(r'^games/game-(?P<game_pk>\d+)/$',
#         GameDetailView.as_view(), name='game_detail'),
# ]
"""
URL configuration for sportspicker project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.urls import path, include

# urlpatterns = [
#
#     path('o/', include('oauth2_provider.urls', namespace='oauth2_provider')),
# ]




# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from datetime import timedelta
from json import loads

from django.urls import reverse
from django.db import models
from django.template.defaultfilters import date as date_format
from django.utils.translation import gettext_lazy as _
from django.utils.encoding import smart_str
from django.utils import timezone

import requests


class Sport(models.Model):
    name = models.CharField(_('Sport Name'), max_length=40, blank=False, default='')
    # api_key = models.CharField(_('API Key'), max_length=40, blank=False, default='')

    class Meta(object):
        verbose_name = _('Sport')
        verbose_name_plural = _('Sports')
        app_label = 'sports'

    def __str__(self):
        return self.name

class League(models.Model):
    name = models.CharField(_('League Name'), max_length=40, blank=False, default='')
    sport = models.ForeignKey('sports.Sport', related_name='leagues', verbose_name=_('Sport'),
                                   on_delete=models.CASCADE)
    # api_key = models.CharField(_('API Key'), max_length=40, blank=False, default='')

    class Meta(object):
        verbose_name = _('League')
        verbose_name_plural = _('Leagues')
        app_label = 'sports'

    def __str__(self):
        return self.name

class Conference(models.Model):
    conference_id = models.CharField(_('conference ID'), max_length=40, blank=False, default='')
    name = models.CharField(_('name'), max_length=64, blank=False, default='')
    league = models.ForeignKey('sports.League', related_name='conferences', verbose_name=_('League'),
                                   on_delete=models.CASCADE)
    class Meta(object):
        verbose_name = _('conference')
        verbose_name_plural = _('conferences')
        app_label = 'sports'

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('sports:conference_detail', args=[self.pk, ])


class Division(models.Model):
    division_id = models.CharField(_('division ID'), max_length=40, blank=False, default='')
    name = models.CharField(_('name'), max_length=32, blank=False, default='')

    conference = models.ForeignKey('sports.Conference', related_name='divisions', verbose_name=_('conference'),
                                   on_delete=models.CASCADE)

    class Meta(object):
        verbose_name = _('division')
        verbose_name_plural = _('divisions')
        app_label = 'sports'

    def __str__(self):
        return self.name


class Venue(models.Model):
    venue_id = models.CharField(_('venue ID'), max_length=40, blank=False, null=False)
    name = models.CharField(_('name'), max_length=128, blank=False, null=False)
    address = models.CharField(_('address'), max_length=128, blank=True, null=True, default=None)
    country = models.CharField(_('country'), max_length=32, blank=True, null=True, default=None)
    city = models.CharField(_('city'), max_length=32, blank=True, null=True, default=None)
    state = models.CharField(_('state'), max_length=32, blank=True, null=True, default=None)
    zip_code = models.CharField(_('ZIP Code'), max_length=10, blank=True, null=True, default=None)

    class Meta(object):
        verbose_name = _('venue')
        verbose_name_plural = _('venues')
        app_label = 'sports'

    def __str__(self):
        # TOTAL HACK
        # Note, this is replacing a unicode long-dash with an ASCII-dash.
        return self.name.replace('–', '-')


class Team(models.Model):
    team_id = models.CharField(_('team ID'), max_length=40, blank=False, default='')
    name = models.CharField(_('name'), max_length=64, blank=True, default='')
    alias = models.CharField(_('alias'), max_length=64, blank=True, default='')
    market = models.CharField(_('market'), max_length=64, blank=True, default='')
    conference = models.ForeignKey('sports.Conference', related_name='teams', verbose_name=_('conference'), null=True, on_delete=models.SET_NULL)
    division = models.ForeignKey('sports.Division', related_name='teams', verbose_name=_('division'), null=True, on_delete=models.SET_NULL)
    venue = models.ForeignKey('sports.Venue', related_name='teams', verbose_name=_('venue'), null=True, on_delete=models.SET_NULL)

    class Meta(object):
        verbose_name = _('team')
        verbose_name_plural = _('teams')
        ordering = ('name', 'division', 'conference',)
        app_label = 'sports'

    def __str__(self):
        return self.full_name

    def get_absolute_url(self):
        return reverse('sports:team_detail', args=[self.pk, ])

    @property
    def short_name(self):
        return self.name

    @property
    def full_name(self):
        return '{0} {1}'.format(self.market, self.name)


class Player(models.Model):
    player_id = models.CharField(max_length=40)
    first_name = models.CharField(_('first name'), max_length=255, blank=False)
    last_name = models.CharField(_('last name'), max_length=255, blank=False)
    team = models.ForeignKey('sports.Team', on_delete=models.CASCADE)
    jersey_number = models.CharField(_('jersey number'), max_length=10, blank=True, null=True)
    height = models.CharField(_('height'), max_length=10, blank=True, null=True)
    weight = models.CharField(_('weight'), max_length=10, blank=True, null=True)
    position = models.CharField(_('position'), max_length=10, blank=True, null=True)
    primary_position = models.CharField(_('primary position'), max_length=10, blank=True, null=True)
    birth_date = models.DateField(_('birth date'), blank=True, null=True)


    class Meta(object):
        verbose_name = _('Player')
        verbose_name_plural = _('Players')
        ordering = ('team', 'last_name', 'first_name',)
        app_label = 'sports'

    def __str__(self):
        return '{first} {last}, {team}'.format(first=self.first_name, last=self.last_name, team=self.team.name)

class Season(models.Model):


    # User Editable Label.
    label = models.CharField(_('label'), max_length=100, blank=True, default=None, null=True)
    season_id = models.CharField(_('season id'), max_length=40, blank=False)
    # These two fields are used to match against the remote api
    name = models.CharField(_('name'), max_length=32, blank=False, default='')
    type = models.CharField(_('type'), max_length=32, blank=False, default='')
    year = models.CharField(_('year'), max_length=4, blank=False, default='')
    start_date = models.DateField(_('start date'), blank=True, null=True)
    end_date = models.DateField(_('end date'), blank=True, null=True)
    league = models.ForeignKey('sports.League', related_name='seasons', verbose_name=_('League'),
                                   on_delete=models.CASCADE)
    # JSON Hash to reduce lookups
    hash = models.CharField(_('hash'), max_length=32, blank=True, null=True)
    class Meta(object):
        verbose_name = _('season')
        verbose_name_plural = _('seasons')
        app_label = 'sports'

    def __str__(self):
        return self.label if self.label else self.name


class GamePeriod(models.Model):
    label = models.CharField(_('label'), max_length=100, blank=True, default=None, null=True)
    period_id = models.CharField(_('game period ID'), max_length=40, blank=True, null=True)
    # TODO: Explore converting this to a PositiveSmallIntegerField
    number = models.PositiveSmallIntegerField(_('number'), blank=False, default=0)
    start_date = models.DateField(_('start date'), blank=True, null=True)
    end_date = models.DateField(_('end date'), blank=True, null=True)
    season = models.ForeignKey('sports.Season', related_name='game_periods', verbose_name=_('season'), blank=True,
                               null=True, on_delete=models.SET_NULL)

    class Meta(object):
        verbose_name = _('game period')
        verbose_name_plural = _('game periods')
        ordering = ('season', 'number',)
        app_label = 'sports'

    @property
    def earliest_game_start(self):
        try:
            return self.games.filter(scheduled__isnull=False).earliest('scheduled').scheduled
        except Game.DoesNotExist:
            return None

    @property
    def latest_game_start(self):
        try:
            return self.games.filter(scheduled__isnull=False).latest('scheduled').scheduled
        except Game.DoesNotExist:
            return None

    @property
    def default_deadline(self):
        try:
            return (self.earliest_game_start - timedelta(days=1)).replace(hour=23, minute=59)
        except TypeError:
            return None

    @property
    def default_expert_deadline(self):
        try:
            return (self.earliest_game_start - timedelta(days=1)).replace(hour=18, minute=00)
        except TypeError:
            return None

    def __str__(self):
        return '{0}: Game:{1}'.format(self.season, self.number)


class Game(models.Model):
    game_id = models.CharField(_('game ID'), max_length=40, blank=False, default='')
    game_period = models.ForeignKey('sports.GamePeriod', related_name='games', verbose_name=_('game period'), null=True, on_delete=models.SET_NULL)
    scheduled = models.DateTimeField(_('scheduled'))
    scored = models.DateTimeField(_('scored'), blank=True, null=True)
    summary = models.TextField(_('summary'), blank=True, null=True)
    coverage = models.CharField(_('coverage'), max_length=32, blank=True, default='')
    home_rotation = models.CharField(_('home rotation'), max_length=32, blank=True, default='')
    away_rotation = models.CharField(_('away rotation'), max_length=32, blank=True, default='')
    home = models.ForeignKey('sports.Team', related_name='game_home', on_delete=models.CASCADE)
    away = models.ForeignKey('sports.Team', related_name='game_away', on_delete=models.CASCADE)
    status = models.CharField(_('status'), max_length=32, blank=True, default='')
    venue = models.ForeignKey('sports.Venue', related_name='games', verbose_name=_('venue'), null=True, on_delete=models.SET_NULL)
    home_points = models.PositiveSmallIntegerField(_('home points'), blank=True, null=True)
    away_points = models.PositiveSmallIntegerField(_('away points'), blank=True, null=True)

    class Meta(object):
        verbose_name = _('game')
        verbose_name_plural = _('games')
        ordering = ['scheduled', ]
        app_label = 'sports'

    def __str__(self):
        # return '{o.away} at {o.home}'.format(o=self)
        return self.full_name_with_date

    def get_absolute_url(self):
        # Returns the "sportsadmin" URL for this game
        return reverse('sports:game_detail', args=[self.pk, ])

    def save(self, **kwargs):
        """
        Ensure that `scored` is set to None if there are no scores and a
        timestamp (or leave as the provided non-None value) if they are set.
        """
        if self.home_points is None and self.away_points is None:
            self.scored = None
        elif self.scored is None:
            self.scored = timezone.now()
        super(Game, self).save(**kwargs)

    @property
    def name(self):
        return '{o.away} at {o.home}'.format(o=self)

    @property
    def game_date(self):
        return date_format(self.scheduled)

    @property
    def short_name(self):
        return '{o.away.short_name} at {o.home.short_name}'.format(o=self)

    @property
    def full_name(self):
        return '{o.away.full_name} at {o.home.full_name}'.format(o=self)

    @property
    def markets_name(self):
        return '{o.away.market} at {o.home.market}'.format(o=self)

    @property
    def full_name_with_date(self):
        return '{o.away.full_name} at {o.home.full_name} ({o.game_date})'.format(o=self)

    @property
    def game_period_name(self):
        return '{o.game_period.number}'.format(o=self)

    @property
    def season_name(self):
        return '{o.game_period.season.season_name}'.format(o=self)

    def get_winner(self):
        if self.is_scored():
            if self.home_points > self.away_points:
                return self.home
            elif self.away_points > self.home_points:
                return self.away
        return None

    def pull_score(self, api_key, format='json'):
        # TODO need to adapt for both NFL and NCAA
        if self.scored is None:
            api_call = 'http(s)://api.sportradar.us/ncaafb-[access_level][version]/[year]/[ncaafb_season]/' \
                       '[ncaafb_season_week]/[away_team]/[home_team]/summary.[format]?api_key=[your_api_key]'
            api_call = api_call.replace('(s)', '')
            season = self.game_period.season
            api_call = api_call.replace('[year]', season.season_name)
            api_call = api_call.replace('[ncaafb_season]', season.season_type)
            api_call = api_call.replace('[ncaafb_season_week]', str(self.game_period.number))
            if 'ncaafb' in api_call:
                api_call = api_call.replace('[access_level][version]', 't1')
            elif 'nfl' in api_call:
                api_call = api_call.replace('[access_level][version]', 'ot1')
            api_call = api_call.replace('[away_team]', self.away.team_id)
            api_call = api_call.replace('[home_team]', self.home.team_id)
            api_call = api_call.replace('[format]', format)
            api_call = api_call.replace('[your_api_key]', api_key)
            result = loads(self.send_request(api_call))
            if (result['status']) == 'closed':
                self.away_points = (result['away_team'])['points']
                self.home_points = (result['home_team'])['points']
                self.summary = result
                self.scored = timezone.now()

                self.save()
                return '{o.away.full_name} {o.away_points}, {o.home.full_name} {o.home_points}'.format(o=self)
            elif (result['status']) == 'complete':
                return 'Game Complete, Scores Being Finalized: {o.away.full_name} at {o.home.full_name}'.format(o=self)
            else:
                return 'Game In Progress: {o.away.full_name} at {o.home.full_name}'.format(o=self)
        else:
            return 'Already Scored on {o.scored}: {o.away.full_name} {o.away_points}, {o.home.full_name} ' \
                   '{o.home_points}'.format(o=self)

    def send_request(self, url):
        try:
            response = requests.get(url=url)
            return smart_str(response.text)
        except requests.exceptions.RequestException:
            raise

    def is_scored(self):
        return not (self.scored is None)


class Broadcast(models.Model):
    broadcast_type = models.CharField(_('type'), max_length=40, blank=False, default='')
    broadcast_address = models.CharField(_('address'), max_length=64, blank=False, default='')
    game = models.ForeignKey('sports.Game', verbose_name=_('game'), on_delete=models.CASCADE)

    class Meta(object):
        verbose_name = _('broadcast')
        verbose_name_plural = _('broadcasts')
        app_label = 'sports'

    def __str__(self):
        return '{o.broadcast_type}:{o.broadcast_address}'.format(o=self)


class GameSummary(models.Model):
    game = models.ForeignKey('sports.Game', related_name='game', on_delete=models.CASCADE)
    home_points = models.PositiveSmallIntegerField(_('home points'), blank=False, default=0)
    away_points = models.PositiveSmallIntegerField(_('away points'), blank=False, default=0)

    class Meta(object):
        verbose_name = _('summary')
        verbose_name_plural = _('summaries')
        app_label = 'sports'
